from ..services.api_client import *
from ..services.cli_handler import *
from ..services.mf_upload import *


def download(file_id=None, file_path=None):
    """This project mainly download files and performs upload instance operations."""
    print("In optima download function")
